﻿using System;
using System.Diagnostics;
using System.Net;
using System.Windows.Forms;
using Ozeki.VoIP;
using SIP_SMS_Example.Windows;
using System.Text;
using Ozeki.Media;

namespace SIP_SMS_Example
{
    public partial class PhoneMain : Form
    {
        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        RegState phoneLineInformation;
        IPhoneCall call;
        bool inComingCall;

        SIPMessageLogger sipMessageLogger;

        public PhoneMain()
        {
            InitializeComponent();
        }

        private void PhoneMain_Load(object sender, EventArgs e)
        {
            Tssl_Ozeki.Text = string.Format("© Copyright {0}, Ozeki Informatics Ltd.", DateTime.Now.Year);

            InitializeSoftPhone();
        }


        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// It is the function of 'IPhoneLine' object representing a phoneline, that publishes changes related to the phoneline. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, RegistrationStateChangedArgs e)
        {
            phoneLineInformation = e.State;
            InvokeGUIThread(() =>
            {
                labelIdentifier.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.State == RegState.RegistrationSucceeded)
                {
                    labelRegStatus.Text = "Online";
                    labelCallStateInfo.Text = "Registration succeeded";
                }
                else
                    labelCallStateInfo.Text = e.State.ToString();
            });
        }

        /// <summary>
        /// The 'IPhoneLine' object representing the phoneline, publishes the incoming call via 'IncommingCall' event. This event is made by this event handler function
		/// in this example that will receive the given call. The call then only needs to be pickup to be accepted or rejected.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
                         {
                             labelCallStateInfo.Text = "Incoming call";
                             labelDialingNumber.Text = String.Format("from {0}", e.Item.DialInfo);
                             call = e.Item;
                             call.CallStateChanged += call_CallStateChanged;
                             inComingCall = true;
                         });
        }

        /// <summary>
        /// This is the event handler that publishes the call status changes of 'IphonCall' object representing the call process. It notifies about the status of the call process. 
        /// For example: Ringing, Incall, Completed, Cancelled ...
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            InvokeGUIThread(() => { labelCallStateInfo.Text = e.State.ToString(); });

            switch (e.State)
            {
                case CallState.Completed:
                    call.CallStateChanged -= call_CallStateChanged;
                    call = null;
                    InvokeGUIThread(() => { labelDialingNumber.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    call.CallStateChanged -= call_CallStateChanged;
                    call = null;
                    break;
            }
        }

        private void SendSMS()
        {
            string url =
                string.Format(@"http://{0}/api?action=sendmessage&username={1}&password={2}&recipient={3}&messagetype=SMS:TEXT&messagedata={4}",
                              Txb_IpAddress.Text, txb_UserName.Text, Txb_Password.Text, Txb_recipient.Text, Txb_SmsText.Text);

            WebRequest request = WebRequest.Create(url);

            request.GetResponse();
        }

        void sipMessageLogger_SIPMessageReceived(object sender, VoIPEventArgs<string> e)
        {
            if (e.Item.Contains(txb_SIPText.Text))
            {
                SendSMS();
            }
        }

        #endregion

        #region Keypad events

        /// <summary>
        /// It initializes calls by a dialed phone number or it accepts or pickup incoming calls.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPickUp_Click(object sender, EventArgs e)
        {
            if (inComingCall)
            {
                inComingCall = false;
                call.Answer();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(labelDialingNumber.Text))
                return;

            if (phoneLineInformation != RegState.RegistrationSucceeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }

            call = softPhone.CreateCallObject(phoneLine, labelDialingNumber.Text);
            call.CallStateChanged += call_CallStateChanged;
            call.Start();
        }

        /// <summary>
        /// If a call is in progress, it breaks the given call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHangUp_Click(object sender, EventArgs e)
        {
            if (call != null)
            {
                inComingCall = false;
                call.HangUp();
                call = null;
            }
            labelDialingNumber.Text = string.Empty;
        }

        /// <summary>
        /// It appends the dialed phone numbers to the phone numbers to be called.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_Click(object sender, EventArgs e)
        {
            var btn = sender as Button;
            if (btn != null)
            {
                if (btn.Text == "0/+")
                {
                    labelDialingNumber.Text += "0";
                }
                else
                    labelDialingNumber.Text += btn.Text.Trim();
            }
        }

        /// <summary>
        /// It sends standard DTMF signals according to RFC 2833.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_MouseDown(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StartDTMFSignal((DtmfNamedEvents)id);
                    }
                }
            }
        }
        /// <summary>
        /// It stops sending standard DTMF signals according to RFC 2833.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPad_MouseUp(object sender, MouseEventArgs e)
        {
            if (call != null && call.CallState == CallState.InCall)
            {
                var btn = sender as Button;
                if (btn != null)
                {
                    int id;

                    if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                    {
                        call.StopDTMFSignal((DtmfNamedEvents)id);
                    }
                }
            }
        }

        #endregion

        #region Helper Functions
        /// <summary>
        /// It initializes a softphone object with SIP PBX access and a SIP account that is needed for using a SIP PBX service. It also registeres this SIP account to the given PBX via an 'IPhoneLine' object that represents a phoneline.
        /// If the registration is successful, we have a call ready softphone. In this example this softphone can be reached by dialing 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
			try
			{
				softPhone = SoftPhoneFactory.CreateSoftPhone(5700, 5750);
				softPhone.IncomingCall += softPhone_IncomingCall;
				SIPAccount sipAccount = new SIPAccount(true, "875", "875", "875", "875", "192.168.115.103", 5060);
                phoneLine = softPhone.CreatePhoneLine(sipAccount);
				phoneLine.RegistrationStateChanged += phoneLine_PhoneLineInformation;
	
				softPhone.RegisterPhoneLine(phoneLine);

                sipMessageLogger = new SIPMessageLogger();
                sipMessageLogger.SIPMessageReceived += (sipMessageLogger_SIPMessageReceived);
                sipMessageLogger.Start();
			}
			catch(Exception ex)
			{
			    var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if(ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
			}
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread. In this way it is possible to modify the control values on the GUI.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }
        #endregion

        #region Menu events
        private void Tsmi_HelpPage_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/p_174-how-to-use-the-c-sip-softphone-source-of-ozeki-voip-sdk-for-creating-a-sip-sms-example-voip.html");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void Tsmi_About_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();

            aboutBox.ShowDialog(this);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        #endregion

        private void PhoneMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            sipMessageLogger.SIPMessageReceived -= (sipMessageLogger_SIPMessageReceived);
            softPhone.Close();
        }
    }
}
